
Library ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_arith.all;


ENTITY BF_I IS
  GENERIC( 
    Input_width : integer := 36;
    Output_width : integer := 36
    );
  PORT(
    S        : IN     std_logic;
    BF1_I1   : IN     std_logic_vector (Input_width-1 DOWNTO 0);
    BF1_I2   : IN     std_logic_vector (Input_width-1 DOWNTO 0);
    BF1_O1  : OUT    std_logic_vector (Output_width-1 DOWNTO 0);
    BF1_O2  : OUT    std_logic_vector (Output_width-1 DOWNTO 0)
    );
  
END BF_I;

ARCHITECTURE rtl OF BF_I IS
  signal BF1_I1_re:     std_logic_vector (Input_width/2-1 downto 0);
  signal BF1_I1_imag:   std_logic_vector (Input_width/2-1 downto 0);
  signal BF1_I2_re:     std_logic_vector (Input_width/2-1 downto 0);
  signal BF1_I2_imag:   std_logic_vector (Input_width/2-1 downto 0);
  signal BF1_O1_re:    std_logic_vector (Output_width/2  downto 0);
  signal BF1_O1_imag:  std_logic_vector (Output_width/2  downto 0);
  signal BF1_O2_re:    std_logic_vector (Output_width/2  downto 0);
  signal BF1_O2_imag:  std_logic_vector (Output_width/2  downto 0);
  
begin
  BF1_I1_re    <= BF1_I1 (Input_width-1 downto Input_width/2);
  BF1_I1_imag  <= BF1_I1 (Input_width/2-1 downto 0);
  BF1_I2_re    <= BF1_I2 (Input_width-1 downto Input_width/2);
  BF1_I2_imag  <= BF1_I2 (Input_width/2-1 downto 0);
  
  BF1_O1_re <= (BF1_I1_re(17) & BF1_I1_re)  + (BF1_I2_re(17) & BF1_I2_re);
  BF1_O2_re <= (BF1_I1_re(17) & BF1_I1_re)  - (BF1_I2_re(17) & BF1_I2_re);
  BF1_O1_imag <= (BF1_I1_imag(17) & BF1_I1_imag) + (BF1_I2_imag(17) & BF1_I2_imag);
  BF1_O2_imag <= (BF1_I1_imag(17) & BF1_I1_imag) - (BF1_I2_imag(17) & BF1_I2_imag);
  
  --Output 1
  BF1_O1 <= BF1_I1 when S ='0' else (BF1_O1_re(Output_width/2 downto 1) & BF1_O1_imag(Output_width/2 downto 1)) ;
  --Output 2
  BF1_O2 <= BF1_I2 when S ='0' else (BF1_O2_re(Output_width/2 downto 1) & BF1_O2_imag(Output_width/2 downto 1));
  
  
END;  


